# NOTE: anything takeable inside a container needs an icon!
init python:
    class Item (object):
        def __init__ (self, name, icon=None, takeable=False):
            self.name = name
            self.tag = name
            self.icon = "icons/{}_%s.png".format (icon) if icon else None
            self.verbs = {}
            self.moved = False
            self.count = 1
            self.stackable = False
            self.room = None
            self.room_pos = None
            self.blued = "blued"
            if takeable:
                self.add_verb ("take", txt="You take the {}.".format (name))

        def __repr__ (self):
            return "Item(name=%r,icon=%r,count=%r)" % (self.name, self.icon, self.count)

        def set_icon (self, icon=None):
            self.icon = "icons/{}_%s.png".format (icon) if icon else None

        def __str__ (self):
            return self.name if self.count == 1 else "{}x {}".format (self.count, self.name)

        def unassign_room (self):
            if self.room:
                self.room.items.remove (self)
                self.room = None
                renpy.hide (self.tag)

        def assign_room (self, r, pos, blued_image=None):
            if isinstance (r, Room):
                r = r.name
            self.room = r if isinstance (r, RealRoom) else store.rooms [r]
            self.room_pos = pos
            self.b_image = blued_image
            self.room.items.append (self)

        def append_to (self, cont, is_initial=False):
            self.unassign_room ()

            if self in cont:
                if self.stackable:
                    self.count += 1
            else:
                cont.append (self)
                # self.count = 1
                if not is_initial:
                    self.moved = True

        def remove_from (self, cont):
            try:
                self.unassign_room ()
                cont.remove (self)
                self.moved = True
            except:
                pass

        def remove_some_from (self, cont, N=1):
            if N >= self.count:
                self.remove_from (cont)
            else:
                self.count -= N

        def add_verb (self, verb, who=None, txt=None, label=None):
            # TODO: other short hands maybe? Or maybe none?
            if verb == "ex":
                verb = "examine"
            self.verbs [verb] = (who, txt, label)

        def look (self, txt):
            self.verbs ["look"] = (None, txt, None)

        def ex (self, txt):
            self.examine (txt)

        def examine (self, txt):
            self.verbs ["examine"] = (None, txt, None)

        def has_verb (self, verb):
            return verb in self.verbs

        def remove_verb (self, verb):
            self.verbs.pop (verb, None)

        def action (self):
            return self

        def __call__ (self, verb, other=None):
            if verb == "examine" and not verb in self.verbs:
                verb = "look"
            if not verb in self.verbs:
                renpy.call ("cant_do_this", self, verb, other)
            else:
                who, txt, label = self.verbs [verb]
                if label:
                    renpy.call (label, self, verb, other)
                elif who:
                    if isinstance (who, basestring):
                        who = eval (who)
                    who (txt)
                else:
                    narrator (txt)
                # TODO: that's a HACK!
                if verb == "take" and not isinstance (self, NPC):
                    self.append_to (store.inventory)

    class Container (Item):
        def __init__ (self, *args, **kwargs):
            super (Container, self).__init__ (*args, **kwargs)
            self.content = []

        def __call__ (self, verb, other=None):
            if verb == "look" or verb == "examine":
                super (Container, self).__call__ (verb, other)
                if not self.content:
                    narrator ("The {} contains nothing of relevance.".format (self.name))
                else:
                    x = [s.name for s in self.content]
                    items = " and ".join ([", ".join (x [:-1]), x[-1]]) if len(x) > 1 else x[0]
                    narrator ("The {} contains: {}.".format (self.name, items))
            elif verb == "take":
                if not self.content:
                    narrator ("The {} is contains nothing of relevance!".format (self.name))
                else:
                    if other:
                        item = other
                    else:
                        narrator ("What do you want to take out of the {}?".format (self.name))
                        x = [(s.name, s) for s in self.content]
                        x.append (("Never mind", ""))
                        item = renpy.display_menu (x)
                    if item:
                        item.remove_some_from (self.content)
                        item.append_to (store.inventory)
                        narrator ("You take the {} out of the {}.".format (item.name, self.name))
            elif verb == "use" and other:
                if isinstance (other, list):
                    for x in other:
                        try:
                            x.remove_some_from (store.inventory)
                        except:
                            pass
                        x.append_to (self.content)
                    x = [x.name for x in other]
                    items = " and ".join ([", ".join (x [:-1]), x[-1]]) if len(x) > 1 else x[0]
                    narrator ("You put {} into the {}.".format (items, self.name))
                else:
                    narrator ("Do you want to put the {} into the {}?".format (other.name, self.name))
                    if renpy.display_menu ([("Yes", True), ("No", False)]):
                        try:
                            other.remove_some_from (store.inventory)
                        except:
                            pass
                        other.append_to (self.content)
                        narrator ("You put the {} into the {}.".format (other.name, self.name))
            else:
                super (Container, self).__call__ (verb, other)

        def put_into (self, item, is_initial=False):
            try:
                item.remove_some_from (store.inventory)
            except:
                pass
            item.append_to (self.content, is_initial)

        def take_out_of (self, item):
            try:
                item.remove_some_from (self.content)
                return True
            except:
                return False

    # Group a bunch of containers, so we can hide stuff in them.
    class ContainerGroup (list):
        def __init__ (self, *members):
            super (ContainerGroup, self).__init__ ()

            for m in members:
                if isinstance (m, Container):
                    self.append (m)
                else:
                    try:
                        self += list (m)
                    except:
                        print "Can't append", m, "to ContainerGroup"

        def put_into (self, item, is_initial=False):
            if not self:
                return

            m = renpy.random.choice (self)
            m.put_into (item, is_initial)

    # Subclass Item for this!
    # Cluestates: "idle", "destroyed", "found", "sent", "results", "informed", "irrelevant"
    class Clue (Item):
        def __init__ (self, name, label, description=None, owner=None, delay=33):
            # super (Clue, self).__init__ (name, icon=name, takeable=True)
            super (Clue, self).__init__ (name, takeable=True)
            self.state = "idle"
            self.owner = owner or npc_peppers
            self.report_label = label
            self.delay = delay
            self.description = description
            if description:
                self.add_verb ("look", label="look_or_examine_clue")
                self.add_verb ("ex", label="look_or_examine_clue")
                self.add_verb ("CSI", label="look_or_examine_clue")
            self.known = False
            self.innocent = True

        def is_visible (self):
            return self.state in ["idle", "found"]

        def __call__ (self, verb, other=None):
            if not self.is_visible ():
                renpy.call ("cant_do_this", self, verb, other)
                # NOTE: This won't be reached anyway.
                return

            if verb == "take" and self.has_verb ("take"):
                narrator ("Do you want to just take the {} or have it secured for evidence?".format (self.name))
                x = [("Just take it with you." if self.innocent else "Don't let it lay around like that!", "take"),
                     ("It might be relevant to the case!", "secure")]
                verb = renpy.display_menu (x)
                if verb != "secure":
                    narrator ("You take the {} and hand it over to your partner.".format (self.name))
                    self.taken ()
                    return

            if verb == "takefrom":
                self.remove_some_from (other.content)
                narrator ("You take the {} from the {} and hand it over to your partner.".format (self.name, other.name))
                self.taken ()
            elif verb == "secure":
                p ("Please secure the {} as evidence.".format (self.name))
                if isinstance (other, Container):
                    self.remove_some_from (other.content)
                    narrator ("{} carefully documents your find, removes it from the {} and puts it into an evidence bag.".format (sk_She, other.name))
                else:
                    narrator ("{} carefully documents your find and puts it into an evidence bag.".format (sk_She))
                sk ("I'll let Peppers have it examined.")
                self.send ()
            else:
                super (Clue, self).__call__ (verb, other)

        # TODO: should we really fail silently?
        def find (self):
            if not self.is_visible ():
                print self.name, ".found() while not visible."
                return
            self.state = "found"

        def is_found (self):
            return self.state == "found"

        def invalidate (self):
            self.state = "irrelevant"
            self.unassign_room ()

        def taken (self):
            if not self.is_visible ():
                print self.name, ".taken() while not visible."
                return
            self.moved = True
            self.state = "destroyed"
            self.unassign_room ()

        def send (self, worklist=True):
            if not self.is_visible ():
                print self.name, ".send() while not visible."
                return
            self.moved = True
            self.state = "sent"
            self.unassign_room ()
            Event (self.owner, self, minutes=self.delay, worklist=worklist)

        def is_being_examined (self):
            return self.state == "sent"

        # TODO: use something like pepper_results.add (self) / coroner_results.add (self) or something.
        def examined (self, queue):
            # Don't report twice on the same Clue.
            if self.state != "sent":
                return

            # TODO: random variations? NPC-specific?
            Character ("Textmessage from {}".format (self.owner.name)) ("Call me about the {}!".format (self.name))
            self.owner.report_queue.append (self)
            self.state = "results"

        def is_ready (self):
            return self.state == "results"

        def informed (self):
            # see Clue.examined ()!
            # Event.dequeue (self.owner, self)
            self.state = "informed"
            store.clues_handled.add (self)
            self.known = True

    class Hint (object):
        # def __init__ (self, short, description):
        def __init__ (self, description):
            self.description = description
            self.known = False

        def got (self):
            store.hints_got.add (self)
            self.known = True

    # Pseudo Items for defining verbs like "look" or "take" for the inventory bar.
    class PseudoItem (object):
        def __init__ (self, verb, icon):
            self.verb = verb
            self.icon = "icons/{}_%s.png".format (icon) if icon else None

        def action (self):
            return self.verb

    # Pseudo Item for exits
    class Exit (Item):
        def __init__ (self, exits_to, desc, leave_txt=None):
            super (Exit, self).__init__ ("exit to {}".format (exits_to))
            self.desc = desc
            self.leave_txt = leave_txt
            self.exits_to = rooms [exits_to] if exits_to else None

        def __call__ (self, verb, other=None):
            if verb == "look" or verb == "examine":
                narrator (self.desc)
            elif verb == "take":
                store.current_room.move_to (self.exits_to, self.leave_txt)
            else:
                super (Exit, self).__call__ (verb, other)

    class Topic (object):
        def __init__ (self, owner, prompt, reaction, trigger=None, persistent=False):
            self.owner = owner
            self.prompt = prompt
            self.trigger = trigger
            self.reaction = reaction
            self.persistent = persistent

        def is_active (self):
            if not self.trigger:
                return True
            return self.trigger.known if not isinstance (self.trigger, basestring) else self.owner.topic_handled (self.trigger)

    # NOTE: suspects don't use the report_queue, staff ignore guilty and so on ...
    class NPC (Item):
        def __init__ (self, name, character, image=None, desc=None, talk=None):
            super (NPC, self).__init__ (name)
            self.character = character
            self.image = image
            if desc:
                self.look (desc)
            if talk:
                self.add_verb ("take", label=talk)
            else:
                self.add_verb ("take", txt="{} isn't talking to you.".format (name))
            self.talks = 0
            self.talk_time = store.talk
            self.known = True
            self.report_queue = []
            self.topics = []
            self.seen_topics = []
            self.handled_topics = []
            # A suspect can be either guilty or not.
            self.guilty = False
            self.broken = False
            self.break_check = None
            self.confession = None
            self.next_talk = None

        def report (self):
            r = self.report_queue.pop (0)
            r.informed ()
            renpy.call (r.report_label, r)

        def add_topic (self, prompt, reaction, trigger=None, persistent=False, endgame=False):
            if reaction not in self.seen_topics:
                if endgame or not store.endgame:
                    self.topics.append (Topic (self, prompt, reaction, trigger, persistent))
                self.seen_topics.append (reaction)

        def block_topic (self, reaction):
            if reaction not in self.seen_topics:
                self.seen_topics.append (reaction)
            for t in self.topics:
                if reaction == t.reaction:
                    self.topics.remove (t)
                    break

        def topic_handled (self, reaction, blocked_ok=True):
            if blocked_ok:
                return reaction in self.seen_topics and not self.topic_queued (reaction)
            else:
                return reaction in self.handled_topics

        def topic_queued (self, reaction):
            for t in self.topics:
                if reaction == t.reaction:
                    return True
            return False

        def set_guilty (self, guilty=True):
            self.guilty = guilty

        def set_confession (self, check, confession):
            self.break_check = check
            self.confession = confession

        def break_down (self):
            if self.guilty:
                store.event_queue ["deadline"] = []
                self.broken = True

        def talk_menu (self):
            choices = []
            if self.report_queue:
                choices.append (("Do you have news for me?", self.report))
            for t in self.topics:
                if t.is_active ():
                    choices.append ((t.prompt, t))
            choices.append (("That's all for now.", ""))
            return choices

        def __call__ (self, verb, other=None):
            if verb == "take":
                # TODO: does that even work?
                renpy.call ("talk_with", self)
            else:
                super (NPC, self).__call__ (verb, other)

label talk_with (whom, phone=False):
    if phone:
        hide pers
    else:
        show expression whom.image as pers
    $ whom.talks += 1
    $ clock += whom.talk_time
    if whom.next_talk:
        call expression whom.next_talk from _call_expression_5
        $ whom.next_talk = None
        if _return == "stop talking":
            return
label .cont:
    if whom.guilty and whom.break_check:
        call expression whom.break_check from _call_expression_6
    if whom.broken:
        if my_gun in inventory:
            "[whom.name] refused to talk to you. You get the uncanny feeling you made a fatal mistake."
            return
        elif whom.confession:
            $ confession.got ()
            jump expression whom.confession

    "What do you want to talk with [whom.name] about?"
    $ choices = whom.talk_menu ()
    $ rv = renpy.display_menu (choices)
    if rv:
        if rv in whom.topics:
            if not rv.persistent:
                $ whom.topics.remove (rv)
            $ rv = rv.reaction
            $ whom.handled_topics.append (rv)
        if isinstance (rv, basestring):
            call expression rv from _call_expression_2
        else:
            $ rv ()
        if _return == "stop talking":
            return
        jump .cont

    p "That's all for now, thank you."
    # whom "Good bye."
    return

label init_items:
    $ examine = PseudoItem ("examine", "eye")
    $ take = PseudoItem ("take", "hand")
    return

label cant_do_this (item, verb, other):
    # $ print item, verb, other
    # TODO: randomize this. Create Variations depending on verb and other too.
    if verb == "look":
        if item.count == 1:
            $ name = item.name.capitalize ()
            "[name]."
        else:
            "[item.count]x [item.name]."
    else:
        "You can't do this to the [item.name] at the moment."
    return

label take_item_from_container (item, v, container):
    $ container ("take", item)
    return

label look_or_examine_clue (clue, verb, o):
    if clue.state == "idle":
        $ clue.find ()
    if verb == "CSI":
        $ lab = "csi_{}".format (clue.name.replace (' ', '_'))
        if renpy.has_label (lab):
            call expression lab pass (clue, verb, o) from _call_expression_3
        else:
            $ lab = "examine_{}".format (clue.name.replace (' ', '_'))
            if renpy.has_label (lab):
                call expression lab pass (clue, verb, o) from _call_expression_4
            else:
                "[clue.description]"
        if clue.is_visible ():
            menu:
                "Should this be secured as evidence?"

                "Yes":
                    $ clue.send ()
                    "You inform Peppers about the trace you found and ask her to analyze it."
                "No":
                    pass
    elif verb == "look":
        "[clue.description]"
    else:
        call expression "examine_{}".format (clue.name.replace (' ', '_')) pass (clue, verb, o) from _call_expression_1

screen content (container):
    style_prefix "content"

    zorder 200

    window:
        has hbox

        vpgrid:
            cols 2
            spacing 2

            draggable True
            mousewheel True

            if len (container.content) > 4:
                scrollbars "vertical"

            for i in container.content:
                imagebutton auto i.icon action SetVariable ("selected", i)

        vbox:
            textbutton "Look" action Return ("look")
            textbutton "Examine" action Return ("examine")
            textbutton "Take" action Return ("takefrom")
            if current_room.name != "my home":
                textbutton "Secure as Evidence" action Return ("secure")
            null
            textbutton "Close" action Return ("close")

style content_button_text:
    color "#f00"
    hover_color "#ff0"

style content_window:
    xsize 400
    ysize 160
    xalign 0.5
    yalign 0.5

    background "images/content_plate.png"

